"""Here is an example service.

Custom metrics are any functions in this module.
"""


def hits():
    """Returns the number of page hits."""

    return 42


def errors():
    """Returns the number of errors."""

    return 4
