<!DOCTYPE html>
<html lang="en">
<head>
 <title>Kube Shields</title>
</head>
<body>
 <h1><a href="/">{{ site_name }}</a></h1>
 <hr />
 {%- for server, services in all_statuses %}
 <div class="cluster">
  <ul>
  {%- for service, checks in services %}
   <li>
    <p>{{ service }}
     {%- for check, url in checks %}
     <a href="/_/{{ service }}/{{ check }}/" title="https://{{ site_name }}/_/{{ service }}/{{ check }}/"><img src="{{ url }}" alt="{{ service }} {{ check }} shield" /></a>
     {%- endfor %}
    </p>
   </li>
  {%- endfor %}
  </ul>
 </div>
 {%- endfor %}
<style type="text/css">
body{
  width: 90%;
  text-align: left;
  margin: 0 auto;
  margin-bottom: 3em;
}
h1{
  display: block;
  text-align: center;
  color: #5C5C5C;
  font-size: 3em;
  font-family: Helvetica, Verdana, sans-serif;
}
a{
  color: inherit;
  text-decoration: none;
}
p{
  margin: 0;
  padding: 0.1em;
}
hr{
  border-color: #ccc;
  width: 55%;
}
ul{
  list-style-type: none;
  margin: 0 auto;
  padding: 0;
  width: 40%;
}
li{
  font: 200 18px/1.4 Helvetica, Verdana, sans-serif;
  border-bottom: 1px solid #ccc;
}
img{
  float: right;
  clear: both;
}
img:after{
  content: '\A';
  white-space: pre;
}
div.cluster{
  padding-bottom: 2em;
}
</style>
</body>
</html>
